<?php

namespace Drupal\vlsuite_block\Controller;

use Drupal\layout_builder\SectionStorageInterface;
use Drupal\layout_builder_restrictions\Controller\ChooseBlockController as ChooseBlockControllerCore;
use Drupal\Core\Render\Element;
use Drupal\Core\Render\Markup;

/**
 * Choose a new block.
 */
class VLSuiteBlockChooseBlockController extends ChooseBlockControllerCore {

  /**
   * {@inheritdoc}
   */
  public function build(SectionStorageInterface $section_storage, $delta, $region) {
    $build = parent::build($section_storage, $delta, $region);
    if (!empty($build['add_block'])) {
      $build['add_block']['#access'] = FALSE;
    }
    // Add wrapper to get filters working @see
    // https://www.drupal.org/project/drupal/issues/3308658#comment-15002205
    $build['#type'] = 'container';
    if (!empty($build['block_categories'])) {
      foreach (Element::children($build['block_categories']) as $category_key) {
        if ($category_key == (string) $this->t('VLSuite: Inline blocks')) {
          $build['block_categories'][$category_key]['#open'] = TRUE;
          $build['block_categories'][$category_key]['#weight'] = -200;
          $build['block_categories'][$category_key]['#title'] = $this->t('VLSuite: Add new block');
        }
        elseif ($category_key == (string) $this->t('VLSuite: Field blocks')) {
          $build['block_categories'][$category_key]['#open'] = FALSE;
          $build['block_categories'][$category_key]['#weight'] = -199;
          $build['block_categories'][$category_key]['#title'] = $this->t('VLSuite: Add field block');
        }
        elseif ($category_key == (string) $this->t('VLSuite: List block (views)')) {
          $build['block_categories'][$category_key]['#open'] = FALSE;
          $build['block_categories'][$category_key]['#weight'] = -198;
          $build['block_categories'][$category_key]['#title'] = $this->t('VLSuite: Add list block');
        }
        else {
          $build['block_categories'][$category_key]['#open'] = FALSE;
        }
        $this->categoryBlockLinksAddIcon($build['block_categories'][$category_key], $category_key);
      }
    }
    return $build;
  }

  /**
   * Generate block icons for category block links.
   *
   * @param array $category
   *   Categrory.
   * @param string $category_key
   *   Categrory key.
   */
  protected function categoryBlockLinksAddIcon(array &$category, $category_key) {
    if (!empty($category['links']) && !empty($category['links']['#links'])) {
      // Default block image.
      $icon_base_path = base_path() . \Drupal::service('extension.list.module')->getPath('vlsuite_block') . '/images/f5f5f2/';
      // @todo alter?!
      // @code $icon_url = \Drupal::service('file_url_generator')->generateString($icon_base_path . 'linuxfoundation.svg'); @endcode
      $icon_url = $icon_base_path . 'puzzlepiece.svg';

      foreach (Element::children($category['links']['#links']) as $delta) {
        if (!empty($category['links']['#links'][$delta]['url'])) {
          if ($category_key == $this->t('VLSuite: Inline blocks')) {
            if ($category['links']['#links'][$delta]['url']->getRouteParameters()['plugin_id'] == 'vlsuite_block_inline_block:vlsuite_text') {
              $icon_url = $icon_base_path . 'pencil.svg';
            }
            elseif ($category['links']['#links'][$delta]['url']->getRouteParameters()['plugin_id'] == 'vlsuite_block_inline_block:vlsuite_image' || $category['links']['#links'][$delta]['url']->getRouteParameters()['plugin_id'] == 'vlsuite_block_inline_block:vlsuite_icon') {
              $icon_url = $icon_base_path . 'linuxfoundation.svg';
            }
            elseif ($category['links']['#links'][$delta]['url']->getRouteParameters()['plugin_id'] == 'vlsuite_block_inline_block:vlsuite_cta') {
              $icon_url = $icon_base_path . 'star.svg';
            }

            elseif ($category['links']['#links'][$delta]['url']->getRouteParameters()['plugin_id'] == 'vlsuite_block_inline_block:vlsuite_webform') {
              $icon_url = $icon_base_path . 'table.svg';
            }
            elseif ($category['links']['#links'][$delta]['url']->getRouteParameters()['plugin_id'] == 'vlsuite_block_inline_block:vlsuite_remote_video' || $category['links']['#links'][$delta]['url']->getRouteParameters()['plugin_id'] == 'vlsuite_block_inline_block:vlsuite_local_video') {
              $icon_url = $icon_base_path . 'youtube.svg';
            }
            elseif ($category['links']['#links'][$delta]['url']->getRouteParameters()['plugin_id'] == 'vlsuite_block_inline_block:vlsuite_paragraph') {
              $icon_url = $icon_base_path . 'puzzlepiece.svg';
            }
          }
          elseif ($category_key == (string) $this->t('VLSuite: Field blocks')) {
            $icon_url = $icon_base_path . 'star.svg';
          }
          elseif ($category_key == (string) $this->t('VLSuite: List block (views)')) {
            $icon_url = $icon_base_path . 'grid.svg';
          }
          $label = $category['links']['#links'][$delta]['title'];
          $category['links']['#links'][$delta]['title'] = Markup::create('<img src="' . $icon_url . '" class="block-link-img" /> ' . '<span class="block-link-label">' . $label . '</span>');
        }
      }
    }
  }

}
